@@ -27,11 +27,10 @@ THE SOFTWARE.
 ################ Import python libraries
 
 import math
-import numpy as np
-import matplotlib as mpl
 import ROOT
 import re
 import copy
+import array
 
 
 ################ Define constants
@@ -50,7 +49,6 @@ class Hist2D(object):
                 raise TypeError("%s does not inherit from TH2" % hist)
         except:
             raise TypeError("%s is not a ROOT object" % hist)
-        array = np.array
         self.nbinsx = nx = hist.GetNbinsX()
         self.nbinsy = ny = hist.GetNbinsY()
         self.binlabelsx = _process_bin_labels([hist.GetXaxis().GetBinLabel(i)
@@ -63,24 +61,35 @@ class Hist2D(object):
         if self.binlabelsy:
             self.nbinsy = ny = self.binlabelsy.index('')
             self.binlabelsy = self.binlabelsy[:ny]
-        self.content = array([array([hist.GetBinContent(i, j)
-                                     for i in range(1, nx + 1)])
-                              for j in range(1, ny + 1)])
-        self.xedges = array([hist.GetXaxis().GetBinLowEdge(i)
-                             for i in range(1, nx + 2)])
-        self.yedges = array([hist.GetYaxis().GetBinLowEdge(i)
-                             for i in range(1, ny + 2)])
-        self.x      = array([(self.xedges[i+1] + self.xedges[i])/2
-                             for i in range(nx)])
-        self.y      = array([(self.yedges[i+1] + self.yedges[i])/2
-                             for i in range(ny)])
-        if title: self.title = _parse_latex(title, replace)
-        else: self.title = _parse_latex(hist.GetTitle(), replace)
-        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
-        else: self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
-        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
-        else: self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
-        self.label = _parse_latex(label, replace)
+        self.entries = hist.GetEntries()
+        self.content = [[hist.GetBinContent(i, j) for i in range(1, nx + 1)]
+                        for j in range(1, ny + 1)]
+        self.xedges = [hist.GetXaxis().GetBinLowEdge(i)
+                             for i in range(1, nx + 2)]
+        self.yedges = [hist.GetYaxis().GetBinLowEdge(i)
+                             for i in range(1, ny + 2)]
+        self.x      = [(self.xedges[i+1] + self.xedges[i])/2
+                             for i in range(nx)]
+        self.y      = [(self.yedges[i+1] + self.yedges[i])/2
+                             for i in range(ny)]
+        self.title  = _parse_latex(title or hist.GetTitle(), replace)
+        self.xlabel = _parse_latex(xlabel or hist.GetXaxis().GetTitle(),replace)
+        self.ylabel = _parse_latex(ylabel or hist.GetYaxis().GetTitle(),replace)
+        self.label  = _parse_latex(label, replace)
+    def _flat_content(self):
+        flatcontent = []
+        for row in self.content:
+            flatcontent += row
+        return flatcontent
+    def __getitem__(self, index):
+        """Return contents of indexth bin: x.__getitem__(y) <==> x[y]"""
+        return self._flat_content()[index]
+    def __len__(self):
+        """Return the number of bins: x.__len__() <==> len(x)"""
+        return len(self._flat_content())
+    def __iter__(self):
+        """Iterate through bins: x.__iter__() <==> iter(x)"""
+        return iter(self._flat_content())
     def contour(self, **kwargs):
         """Draw a contour plot."""
         _load_pyplot()
@@ -115,9 +124,21 @@ class Hist2D(object):
         x = np.hstack([self.x for i in range(self.nbinsy)])
         y = np.hstack([[yval for i in range(self.nbinsx)] for yval in self.y])
         maxvalue = np.max(self.content)
-        sizes = self.content.flatten() / maxvalue * maxsize
+        if maxvalue == 0:
+            maxvalue = 1
+        sizes = np.array(self.content).flatten() / maxvalue * maxsize
         plot = plt.scatter(x, y, sizes, marker='s', **kwargs)
         return plot
+    def TH2F(self, name=""):
+        """Return a ROOT.TH2F object with contents of this Hist2D."""
+        th2f = ROOT.TH2F(name, "",
+                         self.nbinsx, array.array('f', self.xedges),
+                         self.nbinsy, array.array('f', self.yedges))
+        th2f.SetTitle("%s;%s;%s" % (self.title, self.xlabel, self.ylabel))
+        for ix in range(self.nbinsx):
+            for iy in range(self.nbinsy):
+                th2f.SetBinContent(ix + 1, iy + 1, self.content[ix][iy])
+        return th2f
 
 class Hist(object):
     """A container to hold the parameters from a ROOT histogram."""
@@ -132,54 +153,48 @@ class Hist(object):
                 self.__init_TGraph(hist)
             except AttributeError:
                 raise TypeError("%s is not a 1D histogram or TGraph" % hist)
-        if title: self.title = _parse_latex(title, replace)
-        else: self.title = _parse_latex(hist.GetTitle(), replace)
-        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
-        else: self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
-        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
-        else: self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
-        self.label = _parse_latex(label, replace)
+        self.title  = _parse_latex(title or hist.GetTitle(), replace)
+        self.xlabel = _parse_latex(xlabel or hist.GetXaxis().GetTitle(),replace)
+        self.ylabel = _parse_latex(ylabel or hist.GetYaxis().GetTitle(),replace)
+        self.label  = _parse_latex(label, replace)
     def __init_TH1(self, hist):
-        array = np.array
         self.nbins = n = hist.GetNbinsX()
         self.binlabels = _process_bin_labels([hist.GetXaxis().GetBinLabel(i)
                                               for i in range(1, n + 1)])
-        if self.binlabels: # Get rid of extra non-labeled bins
+        if self.binlabels and '' in self.binlabels:
+            # Get rid of extra non-labeled bins
             self.nbins = n = self.binlabels.index('')
             self.binlabels = self.binlabels[:n]
-        self.xedges = array([hist.GetBinLowEdge(i) for i in range(1, n + 2)])
-        self.x      = array([(self.xedges[i+1] + self.xedges[i])/2
-                             for i in range(n)])
-        self.xerr   = array([(self.xedges[i+1] - self.xedges[i])/2
-                             for i in range(n)])
-        self.y      = array([hist.GetBinContent(i) for i in range(1, n + 1)])
-        self.yerr   = array([hist.GetBinError(  i) for i in range(1, n + 1)])
-        self.width  = array([self.xedges[i+1] - self.xedges[i]
-                             for i in range(self.nbins)])
+        self.entries = hist.GetEntries()
+        self.xedges = [hist.GetBinLowEdge(i) for i in range(1, n + 2)]
+        self.x      = [(self.xedges[i+1] + self.xedges[i])/2 for i in range(n)]
+        self.xerr   = [(self.xedges[i+1] - self.xedges[i])/2 for i in range(n)]
+        self.width  = [(self.xedges[i+1] - self.xedges[i])   for i in range(n)]
+        self.y      = [hist.GetBinContent(i) for i in range(1, n + 1)]
+        self.yerr   = [hist.GetBinError(  i) for i in range(1, n + 1)]
+        self.yerr   = [self.yerr, self.yerr]
         self.underflow = hist.GetBinContent(0)
         self.overflow  = hist.GetBinContent(self.nbins + 1)
     def __init_TGraph(self, hist):
-        array = np.array
         self.nbins = n = hist.GetN()
-        self.x, self.y = array([]), array([])
+        self.x, self.y = [], []
         x, y = ROOT.Double(0), ROOT.Double(0)
         for i in range(n):
             hist.GetPoint(i, x, y)
-            self.x = np.append(self.x, x)
-            self.y = np.append(self.y, y)
-        upper = array([hist.GetErrorXhigh(i) for i in xrange(n)])
-        lower = array([hist.GetErrorXlow(i)  for i in xrange(n)])
-        self.xerr = array([lower, upper])
-        upper = array([hist.GetErrorYhigh(i) for i in xrange(n)])
-        lower = array([hist.GetErrorYlow(i)  for i in xrange(n)])
-        self.yerr = array([lower, upper])
-        self.xedges = array([self.x[i] - self.xerr[0][i] for i in xrange(n)])
-        self.xedges = np.append(self.xedges,
-                                [self.x[n - 1] + self.xerr[1][n - 1]])
-        self.width = array([self.xedges[i + 1] - self.xedges[i]
-                            for i in range(n)])
+            self.x.append(copy.copy(x))
+            self.y.append(copy.copy(y))
+        lower = [max(0, hist.GetErrorXlow(i))  for i in xrange(n)]
+        upper = [max(0, hist.GetErrorXhigh(i)) for i in xrange(n)]
+        self.xerr = [lower, upper]
+        lower = [max(0, hist.GetErrorYlow(i))  for i in xrange(n)]
+        upper = [max(0, hist.GetErrorYhigh(i)) for i in xrange(n)]
+        self.yerr = [lower, upper]
+        self.xedges = [self.x[i] - self.xerr[0][i] for i in xrange(n)]
+        self.xedges.append(self.x[n - 1] + self.xerr[1][n - 1])
+        self.width = [self.xedges[i + 1] - self.xedges[i] for i in range(n)]
         self.underflow, self.overflow = 0, 0
         self.binlabels = []
+        self.entries = n
     def __getitem__(self, index):
         """Return contents of indexth bin: x.__getitem__(y) <==> x[y]"""
         return self.y[index]
@@ -194,22 +209,40 @@ class Hist(object):
         return iter(self.y)
     def min(self, threshold = None):
         """Return the y-value of the bottom tip of the lowest errorbar."""
-        try:
-            lower_errors = [self.yerr[0][i] for i in range(self.nbins)]
-        except IndexError:
-            lower_errors = [self.yerr[i] for i in range(self.nbins)]
+        lower_errors = self.yerr[0]
         vals = [self.y[i] - lower_errors[i] for i in range(self.nbins)
                 if (self.y[i] - lower_errors[i]) > threshold]
         if vals:
             return min(vals)
         else:
             return threshold
+    def _av_yerr(self):
+        """Return average between the upper and lower yerr."""
+        return [(self.yerr[0][i] + self.yerr[1][i]) / 2
+                for i in range(self.nbins)]
+    def _prepare_xaxis(self, rotation=0, alignment='center'):
+        """Apply bounds and text labels on x axis."""
+        if self.binlabels is not None:
+            binwidth = (self.xedges[-1] - self.xedges[0]) / self.nbins
+            plt.xticks(self.x, self.binlabels,
+                       rotation=rotation, ha=alignment)
+        plt.xlim(self.xedges[0], self.xedges[-1])
+
+    def _prepare_yaxis(self, rotation=0, alignment='center'):
+        """Apply bound and text labels on y axis."""
+        if self.binlabels is not None:
+            binwidth = (self.xedges[-1] - self.xedges[0]) / self.nbins
+            plt.yticks(self.x, self.binlabels,
+                       rotation=rotation, va=alignment)
+        plt.ylim(self.xedges[0], self.xedges[-1])
+
     def scale(self, factor):
         """
         Scale contents, errors, and over/underflow by the given scale factor.
         """
-        self.y *= factor
-        self.yerr *= factor
+        self.y = [x * factor for x in self.y]
+        self.yerr[0] = [x * factor for x in self.yerr[0]]
+        self.yerr[1] = [x * factor for x in self.yerr[1]]
         self.overflow *= factor
         self.underflow *= factor
     def show_titles(self):
@@ -224,15 +257,16 @@ class Hist(object):
         the left.  This can be useful for histograms with labeled bins.
         """
         self.nbins -= 1
-        self.xedges = self.xedges[:-1]
-        self.x = self.x[:-1]
-        self.width = self.width[:-1]
-        self.y = np.delete(self.y, index)
-        self.xerr = np.delete(self.xerr, index)
-        self.yerr = np.delete(self.yerr, index)
+        self.xedges.pop()
+        self.x.pop()
+        self.width.pop()
+        self.y.pop(index)
+        self.xerr.pop(index)
+        self.yerr[0].pop(index)
+        self.yerr[1].pop(index)
         if self.binlabels:
             self.binlabels.pop(index)
-    def hist(self, **kwargs):
+    def hist(self, label_rotation=0, label_alignment='center', **kwargs):
         """
         Generate a matplotlib hist figure.
 
@@ -242,11 +276,10 @@ class Hist(object):
         _load_pyplot()
         plot = plt.hist(self.x, weights=self.y, bins=self.xedges,
                         label=self.label, **kwargs)
-        if self.binlabels is not None:
-            plt.xticks(np.arange(self.nbins), self.binlabels)
-        plt.xlim(self.xedges[0], self.xedges[-1])
+        self._prepare_xaxis(label_rotation, label_alignment)
         return plot
-    def errorbar(self, xerr=False, yerr=False, **kwargs):
+    def errorbar(self, xerr=False, yerr=False, label_rotation=0,
+                 label_alignment='center', **kwargs):
         """
         Generate a matplotlib errorbar figure.
 
@@ -258,11 +291,10 @@ class Hist(object):
             kwargs['yerr'] = self.yerr
         _load_pyplot()
         errorbar = plt.errorbar(self.x, self.y, label=self.label, **kwargs)
-        if self.binlabels is not None:
-            plt.xticks(np.arange(self.nbins), self.binlabels)
-        plt.xlim(self.xedges[0], self.xedges[-1])
+        self._prepare_xaxis(label_rotation, label_alignment)
         return errorbar
-    def errorbarh(self, xerr=False, yerr=False, **kwargs):
+    def errorbarh(self, xerr=False, yerr=False, label_rotation=0,
+                  label_alignment='center', **kwargs):
         """
         Generate a horizontal matplotlib errorbar figure.
 
@@ -272,11 +304,10 @@ class Hist(object):
         if yerr: kwargs['yerr'] = self.xerr
         _load_pyplot()
         errorbar = plt.errorbar(self.y, self.x, label=self.label, **kwargs)
-        if self.binlabels is not None:
-            plt.yticks(np.arange(self.nbins), self.binlabels)
-        plt.ylim(self.xedges[0], self.xedges[-1])
+        self._prepare_yaxis(label_rotation, label_alignment)
         return errorbar
-    def bar(self, xerr=False, yerr=False, **kwargs):
+    def bar(self, xerr=False, yerr=False, label_rotation=0,
+            label_alignment='center', **kwargs):
         """
         Generate a matplotlib bar figure.
 
@@ -288,8 +319,10 @@ class Hist(object):
         _load_pyplot()
         bar = plt.bar(self.xedges[0:-1], self.y, self.width, label=self.label,
                       **kwargs)
+        self._prepare_xaxis(label_rotation, label_alignment)
         return bar
-    def barh(self, xerr=False, yerr=False, yoffset=0., width=0.8, **kwargs):
+    def barh(self, xerr=False, yerr=False, yoffset=0., width=0.8,
+             label_rotation=0, label_alignment='center', **kwargs):
         """
         Generate a horizontal matplotlib bar figure.
 
@@ -304,16 +337,30 @@ class Hist(object):
         width = self.width * width
         barh = plt.barh(xcontent, self.y, width, label=self.label,
                        **kwargs)
-        if self.binlabels is not None:
-            plt.yticks(np.arange(self.nbins), self.binlabels)
+        self._prepare_yaxis(label_rotation, label_alignment)
         return barh
     def TH1F(self, name=""):
-        """Return a ROOT.TH1F object with the contents of this Hist"""
-        th1f = ROOT.TH1F(name, "", self.nbins, self.xedges)
+        """Return a ROOT.TH1F object with contents of this Hist"""
+        th1f = ROOT.TH1F(name, "", self.nbins, array.array('f', self.xedges))
         th1f.SetTitle("%s;%s;%s" % (self.title, self.xlabel, self.ylabel))
         for i in range(self.nbins):
             th1f.SetBinContent(i + 1, self.y[i])
+            try:
+                th1f.SetBinError(i + 1, (self.yerr[0][i] + self.yerr[1][i]) / 2)
+            except TypeError:
+                th1f.SetBinError(i + 1, self.yerr[i])
         return th1f
+    def TGraph(self, name=""):
+        """Return a ROOT.TGraphAsymmErrors object with contents of this Hist"""
+        x = array.array('f', self.x)
+        y = array.array('f', self.y)
+        xl = array.array('f', self.xerr[0])
+        xh = array.array('f', self.xerr[1])
+        yl = array.array('f', self.yerr[0])
+        yh = array.array('f', self.yerr[1])
+        tgraph = ROOT.TGraphAsymmErrors(self.nbins, x, y, xl, xh, yl, yh)
+        tgraph.SetTitle('%s;%s;%s' % (self.title, self.xlabel, self.ylabel))
+        return tgraph
     def divide(self, denominator):
         """
         Return the simple quotient with errors added in quadrature.
@@ -322,19 +369,25 @@ class Hist(object):
             hist3 = hist1.divide_wilson(hist2) <--> hist3 = hist1 / hist2
         """
         quotient = copy.deepcopy(self)
-        quotient.y = np.array([self[i] / denominator[i]
-                               for i in range(self.nbins)])
-        quotient.yerr = np.array([
-            quotient[i] * math.sqrt((self.yerr[i] / self[i]) ** 2 +
-                                    (denominator.yerr[i] / denominator[i])** 2)
-            for i in range(self.nbins)])
+        num_yerr = self._av_yerr()
+        den_yerr = denominator._av_yerr()
+        quotient.yerr = [0. for i in range(self.nbins)]
+        for i in range(self.nbins):
+            if denominator[i] == 0:
+                quotient.y[i] = 0.
+            else:
+                quotient.y[i] = self[i] / denominator[i]
+                quotient.yerr[i] = quotient[i]
+                quotient.yerr[i] *= math.sqrt((num_yerr[i] / self[i]) ** 2 +
+                                       (den_yerr[i] / denominator[i]) ** 2)
+        quotient.yerr = [quotient.yerr, quotient.yerr]
         return quotient
     def divide_wilson(self, denominator):
         """Return an efficiency plot with Wilson score interval errors."""
         eff, upper_err, lower_err = wilson_interval(self.y, denominator.y)
         quotient = copy.deepcopy(self)
-        quotient.y = np.array(eff)
-        quotient.yerr = np.array([lower_err, upper_err])
+        quotient.y = eff
+        quotient.yerr = [lower_err, upper_err]
         return quotient
     def __div__(self, denominator):
         return self.divide(denominator)
@@ -395,7 +448,7 @@ class HistStack(object):
             del kwargs['label']
         self.hists.append(hist)
         self.kwargs.append(kwargs)
-    def hist(self, **kwargs):
+    def hist(self, label_rotation=0, **kwargs):
         """
         Make a matplotlib hist plot.
 
@@ -555,9 +608,16 @@ class RootFile:
     def cd(self, directory=''):
         """Make directory the current directory."""
         self.file.cd(directory)
-    def get(self, object_name):
+    def get(self, object_name, path=None):
         """Return a Hist object from the given path within this file."""
-        return Hist(self.file.Get(object_name))
+        if path:
+            roothist = self.file.GetDirectory(path).Get(object_name)
+        else:
+            roothist = self.file.Get(object_name)
+        try:
+            return Hist2D(roothist)
+        except TypeError:
+            return Hist(roothist)
 
 def ls(directory=None):
     """Return a python list of ROOT object names from the given directory."""
@@ -585,9 +645,11 @@ def get(object_name):
 ################ Define additional helping functions
 
 def _load_pyplot():
-    global plt, transforms
+    global plt, transforms, np
     import matplotlib.pyplot as plt
     import matplotlib.transforms as transforms
+    import numpy as np
+    import matplotlib as mpl
 
 def _parse_latex(string, replacements):
     """
@@ -633,102 +695,6 @@ def wilson_interval(numerator_array, denominator_array):
         lower_err.append(+p - 1/(1 + 1/d) * (t - s))
     return eff, upper_err, lower_err
 
-def axes_broken_y(axes, upper_frac=0.5, break_frac=0.02, ybounds=None,
-                  xlabel=None, ylabel=None):
-    """
-    Replace the current axes with a set of upper and lower axes.
-
-    The new axes will be transparent, with a breakmark drawn between them.  They
-    share the x-axis.  Returns (upper_axes, lower_axes).
-
-    If ybounds=[ymin_lower, ymax_lower, ymin_upper, ymax_upper] is defined,
-    upper_frac will be ignored, and the y-axis bounds will be fixed with the
-    specified values.
-    """
-    def breakmarks(axes, y_min, y_max, xwidth=0.008):
-        x1, y1, x2, y2 = axes.get_position().get_points().flatten().tolist()
-        segment_height = (y_max - y_min) / 3.
-        xoffsets = [0, +xwidth, -xwidth, 0]
-        yvalues  = [y_min + (i * segment_height) for i in range(4)]
-        # Get color of y-axis
-        for loc, spine in axes.spines.iteritems():
-            if loc  == 'left':
-                color = spine.get_edgecolor()
-        for x_position in [x1, x2]:
-            line = mpl.lines.Line2D(
-                [x_position + offset for offset in xoffsets], yvalues,
-                transform=plt.gcf().transFigure, clip_on=False,
-                color=color)
-            axes.add_line(line)
-    _load_pyplot()
-    # Readjust upper_frac if ybounds are defined
-    if ybounds:
-        if len(ybounds) != 4:
-            print "len(ybounds) != 4; aborting..."
-            return
-        ymin1, ymax1, ymin2, ymax2 = [float(value) for value in ybounds]
-        data_height1, data_height2 = (ymax1 - ymin1), (ymax2 - ymin2)
-        upper_frac = data_height2 / (data_height1 + data_height2)
-    x1, y1, x2, y2 = axes.get_position().get_points().flatten().tolist()
-    width = x2 - x1
-    lower_height = (y2 - y1) * ((1 - upper_frac) - 0.5 * break_frac)
-    upper_height = (y2 - y1) * (upper_frac - 0.5 * break_frac)
-    upper_bottom = (y2 - y1) - upper_height + y1
-    lower_axes = plt.axes([x1, y1, width, lower_height], axisbg='None')
-    upper_axes = plt.axes([x1, upper_bottom, width, upper_height],
-                          axisbg='None', sharex=lower_axes)
-    # Erase the edges between the axes
-    for loc, spine in upper_axes.spines.iteritems():
-        if loc == 'bottom':
-            spine.set_color('none')
-    for loc, spine in lower_axes.spines.iteritems():
-        if loc == 'top':
-            spine.set_color('none')
-    upper_axes.get_xaxis().set_ticks_position('top')
-    lower_axes.get_xaxis().set_ticks_position('bottom')
-    plt.setp(upper_axes.get_xticklabels(), visible=False)
-    breakmarks(upper_axes, y1 + lower_height, upper_bottom)
-    # Set ylims if ybounds are defined
-    if ybounds:
-        lower_axes.set_ylim(ymin1, ymax1)
-        upper_axes.set_ylim(ymin2, ymax2)
-        lower_axes.set_autoscaley_on(False)
-        upper_axes.set_autoscaley_on(False)
-        upper_axes.yaxis.get_label().set_position((0, 1 - (0.5 / (upper_frac/(1+break_frac)))))
-        lower_axes.yaxis.get_label().set_position((0, 0.5 / ((1 - upper_frac)/(1+break_frac))))
-    # Make original axes invisible
-    axes.set_xticks([])
-    axes.set_yticks([])
-    for loc, spine in axes.spines.iteritems():
-        spine.set_color('none')
-    return upper_axes, lower_axes
-
-## def prepare_efficiency(axes, lower_bound=0.69):
-##     """
-##     Set up an efficiency figure with breakmarks to indicate a suppressed zero.
-
-##     The y-axis limits are set to (lower_bound, 1.0), as appropriate for an
-##     efficiency plot, and autoscaling is turned off.
-##     """
-##     upper_axes, lower_axes = axes_broken_y(axes, upper_frac=0.97)
-##     lower_axes.set_yticks([])
-##     upper_axes.set_ylim(lower_bound, 1.)
-##     upper_axes.set_autoscaley_on(False)
-##     return upper_axes, lower_axes
-## ## ##     def breakmarks(axes, position):
-## ## ##         plt.axes(axes)
-## ## ##         xwidth = 0.005 * (plt.xlim()[1] - plt.xlim()[0])
-## ## ##         ywidth = 0.001
-## ## ##         if position == 'bottom': j = 0
-## ## ##         if position == 'top': j = 1
-## ## ##         for i in range(2):
-## ## ##             line = mpl.lines.Line2D(
-## ## ##                 [plt.xlim()[i] - xwidth, plt.xlim()[i] + xwidth],
-## ## ##                 [plt.ylim()[j] - ywidth, plt.ylim()[j] + ywidth],
-## ## ##                 linewidth=1, color='black', clip_on=False)
-## ## ##             axes.add_line(line)
-
-
 if __name__ == "__main__":
     # no doctest stuff yet implemented; maybe in future it will be
     import doctest
