@@ -39,6 +39,8 @@ import math
 import os
 import re
 import tempfile
+import copy
+import root2matplotlib as r2m
 
 try:
     import multiprocessing as multi
@@ -75,74 +77,31 @@ up by running 'cmsenv'."""
 default_styles=r"""\
 import ROOT # allows access to ROOT colors (e.g. ROOT.kRed)
 
-##############################################################################
-######## Instructions ########################################################
 
-## If you place this file in your home directory as '~/.rootplot', then
-## it will be automatically loaded whenever you call rootplot.  Otherwise,
-## just include it in the arguments to rootplot to activate it.
-
-## The program will generate default values, then overwrite them with any
-## values included in this file, so feel free to comment out any portions
-## that you don't want to deal with.
-
-##############################################################################
-######## Handling of LaTeX Expressions in matplotlib #########################
-
-## In the replace list below, all text from the first column will be replaced
-## with the corresponding text in the second column for titles and labels;
-## this is necessary to correctly transform ROOT's special LaTeX sytax into
-## proper LaTeX sytax for use by matplotlib.
-
-## By default, matplotlib will render only symbols between $'s as TeX, but if
-## you enable the 'text.usetex' matplotlibrc setting, then everything is handled
-## by the LaTeX engine on your system, in which case you can go wild with TeX.
-
-replace = [
-    # some defaults that should work for most cases
-    (' pt '    , r' $p_\mathrm{T}$ '),
-    ('pT '     , r'$p_\mathrm{T}$ '),
-    (' pT'     , r' $p_\mathrm{T}$'),
-    ('p_{T}'   , r'$p_\mathrm{T}$'),
-    ('#eta'    , r'$\eta$'),
-    ('#phi'    , r'$\phi$'),
-    ('fb^{-1}' , r'$\mathrm{fb}^{-1}$'),
-    ('pb^{-1}' , r'$\mathrm{pb}^{-1}$'),
-    ('<'       , r'$<$'),
-    ('>'       , r'$>$'),
-    ('#'       , r''),
-    ]
-
-## If you include 'use_regexp' as the first item, the patterns to be replaced
-## will function as regular expressions using python's re module rather than
-## as simple text.  The example below turn's ROOT's superscript and subscript
-## syntax into LaTeX:
-
-## replace = [
-##     ('use_regexp', True),
-##     (r'\^\{(.*)\}', r'$^{\1}$'),
-##     (r'\_\{(.*)\}', r'$_{\1}$'),
-## ]
+## Place this file in your home directory as '~/.rootplot', and
+## it will be automatically loaded whenever you call rootplot.
+## Otherwise, include it in the arguments to rootplot to use it.
+## Comment-out/delete sections you don't want to deal with.
 
 ##############################################################################
 ######## Advanced Operation ##################################################
 
-## For more complex needs, you may want to specify a list of files along with
-## more descriptive names for the legend using the filenames option.  When
-## enabled, any filenames on the command line will be ignored.
-
+## You may specify a list of files along with names for the legend
+## When enabled, any filenames on the command line will be ignored.
 ## filenames = [
 ##     ('histTTbar.root', r'$\bar{t}t$'),
 ##     ('histZmumu.root', r'$Z\rightarrow\mu\mu$'),
 ##     ]
 
-## If empty, all factors set to 1; otherwise, requires list at least nfiles long
-file_scale_factors = []
+## All scale factor are 1 by default; specify different scale factors here
+file_scale_factors = []  # must be as long as file list
 
-## Any of the following lists left empty will be filled according to the
-## command-line options.  If filled, they must be at least as long as filenames.
-## To have an entry default to the command-line options, set the value to None.
-linecolors = []
+##############################################################################
+######## Global Style Options ################################################
+
+## Empty lists below are filled according to command-line options.
+## If lists are non-empty, they must be at least as long as filenames.
+linecolors = []           # list of 3-tuples of (R,G,B) values from 0 to 255
 fillcolors = []
 markercolors = []
 markersizes = []          # in points
@@ -154,8 +113,9 @@ plotstyles_mpl = []       # 'bar', 'hist', 'errorbar', 'stack'
 alphas_mpl = []           # transparencies for fills (value from 0 to 1)
 errorbarcolors_mpl = []   # color for bars around the central value
 
-##############################################################################
-######## Global Style Options ################################################
+## A short label can be placed on all plots
+label_text = ''
+label_location = (0.15,0.85) # x,y from 0 to 1
 
 ## Colors can be (r, g, b) tuples from 0 to 1 or 0 to 255, ROOT color constants
 ## (ROOT.kBlue or 600), or any matplotlib color specification if matplotlib is
@@ -215,6 +175,17 @@ ratio_cutoff = 10.    # Ignore bins this many times the average
 top_padding_factor = 1.2
 top_padding_factor_log = 5.
 
+#### Plotting options based on histogram names
+####   ex: set "logy" : ["_pt"] to make plots with _pt in the name be logy
+options_by_histname = {"norm" : [""],   # area normalize
+                       "logx" : [""],
+                       "logy" : [""],
+                       "zero" : [""],
+                       "efficiency" : [""],
+                       "overflow" : [""],
+                       "underflow" : [""],
+                       "ratio" : [""]}
+
 ##############################################################################
 ######## ROOT Output #########################################################
 
@@ -244,6 +215,9 @@ numbering_align_root = 33   # Right-top adjusted
 numbering_x_root = 0.97     # Fraction of canvas width
 numbering_y_root = 0.985    # Fraction of canvas height
 
+#### Define style for TGraph objects
+draw_graph = 'alp'
+
 ##############################################################################
 ######## matplotlib Output ###################################################
 
@@ -254,6 +228,48 @@ numbering_va_mpl = 'top'
 numbering_x_mpl = 0.98       # Fraction of canvas width
 numbering_y_mpl = 0.98       # Fraction of canvas height
 
+#### Rotation for text x-axis labels
+xlabel_rotation = -15
+xlabel_alignment = 'left'
+
+#### Matplotlib options for the label on all plots
+label_align = 'left'         # 'left', 'center' , or 'right'
+label_fontsize = 'medium'    # size in points, 'small', 'medium, 'large', ...
+
+#### Convert ROOT symboles to proper LaTeX, for matplotlib plotting ####
+## By default, matplotlib renders only symbols between $'s as TeX, but if
+## you enable the 'text.usetex' matplotlibrc setting, then everything is handled
+## by the LaTeX engine on your system, in which case you can go wild with TeX.
+
+## ROOT-type strings on left get replaced with proper
+## latex symbols on right for matplotlib plotting
+replace = [
+    # some defaults that should work for most cases
+    (' pt '    , r' $p_\mathrm{T}$ '),
+    ('pT '     , r'$p_\mathrm{T}$ '),
+    (' pT'     , r' $p_\mathrm{T}$'),
+    ('p_{T}'   , r'$p_\mathrm{T}$'),
+    ('E_{T}'   , r'$E_\mathrm{T}$'),
+    ('#eta'    , r'$\eta$'),
+    ('#phi'    , r'$\phi$'),
+    ('fb^{-1}' , r'$\mathrm{fb}^{-1}$'),
+    ('pb^{-1}' , r'$\mathrm{pb}^{-1}$'),
+    ('<'       , r'$<$'),
+    ('>'       , r'$>$'),
+    ('#'       , r''),
+    ]
+
+## If you include 'use_regexp' as the first item, the patterns to be replaced
+## will function as regular expressions using python's re module rather than
+## as simple text.  The example below turn's ROOT's superscript and subscript
+## syntax into LaTeX:
+
+## replace = [
+##     ('use_regexp', True),
+##     (r'\^\{(.*)\}', r'$^{\1}$'),
+##     (r'\_\{(.*)\}', r'$_{\1}$'),
+## ]
+
 ##############################################################################
 ######## HTML Output #########################################################
 
@@ -266,23 +282,7 @@ stylesheet = 'body{padding: 10px; background: #ccccff; font-family: Georgia} img
 """
 
 
-######## Define classes and generators #######################################
-
-class RootFile:
-    """A wrapper for TFiles, allowing quick access to the name and Get."""
-    def __init__(self, file_name, name=None):
-        self.name = name
-        if name == None:
-            self.name = file_name[:-5]
-        self.file = ROOT.TFile(file_name, 'read')
-        if self.file.IsZombie():
-            print "Error opening %s, exiting..." % file_name
-            sys.exit(1)
-    def Get(self, object_name, path=None):
-        if path:
-            return self.file.GetDirectory(path).Get(object_name)
-        else:
-            return self.file.Get(object_name)
+######## Define generator for counting #######################################
 
 def counter_generator():
     """Incremement the counter used to number plots."""
@@ -293,7 +293,6 @@ def counter_generator():
 next_counter = counter_generator().next
 
 
-
 ######## These functions are the meat of this program #########################
 
 #### A recursive function to drill down through directories
@@ -315,7 +314,7 @@ def process_directory(path, files, options):
         matches_path = options.regex.search(new_path)
         dimension = 0
         if ('TH1' in classname or
-            #'TGraph' in classname or
+            'TGraph' in classname or
             classname == 'TProfile'):
             dimension = 1
         if ('TH2' in classname or classname == 'TProfile2D'): dimension = 2
@@ -348,151 +347,24 @@ def process_hist(path, name, counter, options):
     else:
         process_hist_root(path, name, counter, options)
 
-def process_hist_root(path, name, counter, options):
-    """Overlay all the instances of this plot in ROOT and apply the options."""
-    files = open_rootfiles(options.filenames)
-    options = process_local_options(options, name)
-    global canvas
-    canvas = ROOT.TCanvas()
-    if options.ratio:
-        pads = divide_canvas(canvas, options.ratio_fraction)
-        pads[0].cd()
-    else:
-        pads = [canvas]
-    hists, histmax = [], None
-    first_draw = True
-    stack = ROOT.THStack('fullst%.3i' % counter, "")
-    for i, file in enumerate(files):
-        hist = file.file.GetDirectory(path).Get(name)
-        if not hist: continue
-        if hist.InheritsFrom('TH1'):
-            dimension = hist.GetDimension()
-        else:
-            dimension = 1
-        title, xlabel, ylabel = get_labels(hist, options)
-        hist.Scale(options.file_scale_factors[i])
-        if options.area_normalize:
-            ylabel = options.area_normalized_title
-        if type(options.normalize) is float:
-            ylabel = options.scale_normalized_title % options.normalize
-            hist.Scale(options.normalize)
-        if type(options.normalize) is int:
-            filename = files[options.normalize - 1].name
-            ylabel = options.file_normalized_title % filename
-        hist.SetTitle("%s;%s;%s" % (title, xlabel, ylabel))
-        hists.append(hist)
-    for hist in hists:
-        if options.area_normalize:
-            integral = hist.Integral()
-            if integral: hist.Scale(1. / integral)
-        if type(options.normalize) is float:
-            hist.Scale(options.normalize)
-        if type(options.normalize) is int:
-            nbins = hist.GetNbinsX()
-            lowbin, highbin = 1, nbins + 1
-            normhist = hists[options.normalize - 1]
-            if options.range:
-                xedges = [normhist.GetXaxis().GetBinLowEdge(i)
-                          for j in range(1, nbins + 1)]
-                lowbin, highbin = parse_range(xedges, options.range)
-            integral = normhist.Integral(lowbin, highbin)
-            if integral:
-                hist.Scale(integral / hist.Integral(lowbin, highbin))
-        if 'stack' in options.drawcommands_root[i]:
-            stack.Add(hist)
-        histmax = max(histmax, hist.GetMaximum(), stack.GetMaximum())
-    if options.efficiency:
-        histmax = 1. / options.top_padding_factor
-    stack.SetTitle("%s;%s;%s" % (title, xlabel, ylabel))
-    if options.gridx or options.grid:
-        for pad in pads:
-            pad.SetGridx(not pad.GetGridx())
-    if options.gridy or options.grid:
-        pads[0].SetGridy(not pads[0].GetGridy())
-    legend = ROOT.TLegend(*parse_legend_root(options))
-    for com in options.drawcommands_root:
-        if 'stack' in com:
-            first_draw = prep_first_draw(stack, histmax, options)
-            stack.Draw(options.draw)
-            break
-    for i, (file, hist) in enumerate(zip(files, hists)):
-        hist.SetLineColor(options.linecolors[i])
-        hist.SetFillColor(options.fillcolors[i])
-        hist.SetMarkerColor(options.markercolors[i])
-        hist.SetFillStyle(options.fillstyles_root[i])
-        hist.SetMarkerStyle(options.markers_root[i])
-        hist.SetMarkerSize(options.markersizes[i])
-        if dimension == 1:
-            nbins = hist.GetNbinsX()
-            if options.efficiency_from:
-                denom = file.file.Get(options.efficiency_from)
-                num = [hist.GetBinContent(j) for j in range(1, nbins + 1)]
-                den = [denom.GetBinContent(j) for j in range(1, nbins + 1)]
-                eff, up, down = wilson(num, den)
-                for j in range(nbins):
-                    hist.SetBinContent(j + 1, eff[j])
-                    hist.SetBinError(j + 1, max(up[j], down[j]))
-            if options.overflow:
-                hist.AddBinContent(nbins, hist.GetBinContent(nbins + 1))
-            if options.underflow:
-                hist.AddBinContent(1, hist.GetBinContent(0))
-            if 'stack' not in options.drawcommands_root[i]:
-                if first_draw:
-                    first_draw = prep_first_draw(hist, histmax, options)
-                    hist.Draw(options.drawcommands_root[i])
-                else:
-                    hist.Draw("same " + options.drawcommands_root[i])
-        else:
-            hist.Draw(options.draw2D)
-        legend.AddEntry(hist, file.name)
-    if options.ratio and dimension == 1:
-        pads[1].cd()
-        stack_ratio = plot_ratio_root(hists, options)
-        pads[0].cd()
-    if options.logx:
-        for pad in pads:
-            pad.SetLogx(True)
-    if options.logy:
-        for pad in pads:
-            pad.SetLogy(True)
-    if options.numbering:
-        display_page_number(counter, options)
-    if options.overflow and dimension == 1:
-        display_overflow(stack, hist)
-    if options.underflow and dimension == 1:
-        display_underflow(stack, hist)
-    if options.legend and dimension == 1:
-        legend.Draw()
-    save_plot(options.plot_dir, path, name, options.ext, counter)
-
 #### This is where all the plotting actually happens
-def process_hist_mpl(path, name, counter, options):
-    """Overlay all the instances of this plot and output using matplotlib."""
+def process_hist(path, name, counter, options):
+    """Overlay all the instances of this plot."""
     files = open_rootfiles(options.filenames)
     options = process_local_options(options, name)
-    fig = plt.figure(1, figsize=parse_size(options.size))
-    plt.clf() # clear figure
-    axes, hists, = plt.axes(), []
-    axes_list = [axes, axes]
-    if options.ratio:
-        axes_list = divide_axes(axes, options.ratio_fraction)
-        axes = axes_list[0]
-        fig.sca(axes)
-    fullstack, barstack, histmax = r2m.HistStack(), r2m.HistStack(), None
-    allempty = True
+    hists = []
     #### Apply options to hist from each file
     for i, file in enumerate(files):
-        roothist = file.Get(name, path)
-        if roothist.InheritsFrom('TH1') and roothist.GetEntries() > 0:
-            allempty = False
-        if roothist.InheritsFrom('TGraph'):
-            allempty = False
-        if roothist.InheritsFrom('TH1'):
+        roothist = file.file.GetDirectory(path).Get(name)
+        if not roothist:
+            hists.append(None)
+            continue
+        isTGraph = not roothist.InheritsFrom('TH1')
+        dimension = 1
+        if not isTGraph:
             dimension = roothist.GetDimension()
-        else:
-            dimension = 1
+            roothist.Scale(options.file_scale_factors[i])
         title, xlabel, ylabel = get_labels(roothist, options)
-        roothist.Scale(options.file_scale_factors[i])
         if options.normalize:
             if type(options.normalize) is int:
                 file_name = files[options.normalize - 1].name
@@ -509,7 +381,7 @@ def process_hist_mpl(path, name, counter, options):
         if options.efficiency_from:
             denom = r2m.Hist(file.file.Get(options.efficiency_from))
             hist.y, up, down = wilson(hist.y, denom.y)
-            hist.yerr = np.array([up, down])
+            hist.yerr = [down, up]
         if dimension == 1:
             if options.overflow:
                 hist.y[-1] += hist.overflow
@@ -534,6 +406,131 @@ def process_hist_mpl(path, name, counter, options):
                         hist.scale(numer / denom)
                 else:
                     hist.scale(options.normalize)
+        hists.append(hist)
+    if options.mpl:
+        process_hist_mpl(path, name, counter, options, hists)
+    else:
+        process_hist_root(path, name, counter, options, hists,
+                          title, xlabel, ylabel, isTGraph)
+
+
+def process_hist_root(path, name, counter, options, hists,
+                      title, xlabel, ylabel, isTGraph):
+    """Overlay all the instances of this plot in ROOT and apply the options."""
+    global canvas
+    canvas = ROOT.TCanvas()
+    if options.ratio and not isTGraph:
+        pads = divide_canvas(canvas, options.ratio_fraction)
+        pads[0].cd()
+    else:
+        pads = [canvas]
+    histmax, first_draw, roothists = None, True, []
+    multigraph = ROOT.TMultiGraph()
+    stack = ROOT.THStack('fullst%.3i' % counter,
+                         '%s;%s;%s' % (title, xlabel, ylabel))
+    for i, hist in enumerate(hists):
+        if hist:
+            if (type(hist) is r2m.Hist and not isTGraph and 
+                'stack' in options.drawcommands_root[i]):
+                stack.Add(hist.TH1F())
+            if stack.GetHists():
+                histmax = max(histmax, max(hist), stack.GetMaximum())
+            else:
+                histmax = max(histmax, max(hist))
+    dimension = 1
+    if type(hist) == r2m.Hist2D:
+        dimension = 2
+    if options.efficiency:
+        histmax = 1. / options.top_padding_factor
+    if options.gridx or options.grid:
+        for pad in pads:
+            pad.SetGridx(not pad.GetGridx())
+    if options.gridy or options.grid:
+        pads[0].SetGridy(not pads[0].GetGridy())
+    legend = ROOT.TLegend(*parse_legend_root(options))
+    for com in options.drawcommands_root:
+        if 'stack' in com:
+            first_draw = prep_first_draw(stack, histmax, options)
+            stack.Draw(options.draw)
+            break
+    for i, hist in enumerate(hists):
+        if not hist: continue
+        if isTGraph:
+            roothist = hist.TGraph()
+        elif type(hist) is r2m.Hist:
+            roothist = hist.TH1F()
+        else:
+            roothist = hist.TH2F()
+        roothist.SetTitle("%s;%s;%s" % (title, xlabel, ylabel))
+        roothist.SetLineColor(options.linecolors[i])
+        roothist.SetFillColor(options.fillcolors[i])
+        roothist.SetMarkerColor(options.markercolors[i])
+        roothist.SetFillStyle(options.fillstyles_root[i])
+        roothist.SetMarkerStyle(options.markers_root[i])
+        roothist.SetMarkerSize(options.markersizes[i])
+        roothists.append(roothist)
+        if isTGraph:
+            multigraph.Add(roothist)
+        elif dimension == 1:
+            if 'stack' not in options.drawcommands_root[i]:
+                if first_draw:
+                    first_draw = prep_first_draw(roothist, histmax, options)
+                    roothist.Draw(options.drawcommands_root[i])
+                else:
+                    roothist.Draw("same " + options.drawcommands_root[i])
+        else:
+            roothist.Draw(options.draw2D)
+        legend.AddEntry(roothist, options.filenames[i])
+    if isTGraph:
+        multigraph.Draw(options.draw_graph)
+    if options.ratio and dimension == 1 and not isTGraph:
+        pads[1].cd()
+        stack_ratio = plot_ratio_root(roothists, options)
+        pads[0].cd()
+    if options.logx:
+        for pad in pads:
+            pad.SetLogx(True)
+    if options.logy:
+        for pad in pads:
+            pad.SetLogy(True)
+    if options.numbering:
+        display_page_number(counter, options)
+    if roothist.InheritsFrom('TH1'):
+        if options.overflow:
+            display_overflow(stack, roothist)
+        if options.underflow:
+            display_underflow(stack, roothist)
+    if options.legend and dimension == 1:
+        legend.Draw()
+    if options.label_text:
+        a_label_ymax = min(options.label_location[1]+0.05, 1)
+        a_label_xmax = min(options.label_location[0]+0.30, 1)
+        a_label = ROOT.TPaveText(options.label_location[0],
+                                 options.label_location[1],
+                                 a_label_xmax,a_label_ymax,"NDC BR")
+        a_label.AddText(options.label_text)
+        a_label.SetBorderSize(0)
+        a_label.SetFillStyle(0)
+        a_label.Draw()
+    save_plot(options.plot_dir, path, name, options.ext, counter)
+
+
+def process_hist_mpl(path, name, counter, options, hists):
+    """Overlay all the instances of this plot and output using matplotlib."""
+    fig = plt.figure(1, figsize=parse_size(options.size))
+    plt.clf() # clear figure
+    axes = plt.axes()
+    axes_list = [axes, axes]
+    if options.ratio:
+        axes_list = divide_axes(axes, options.ratio_fraction)
+        axes = axes_list[0]
+        fig.sca(axes)
+    fullstack, barstack = r2m.HistStack(), r2m.HistStack()
+    histmax, allempty = None, True
+    for i, hist in enumerate(hists):
+        if hist and hist.entries:
+            allempty = False
+        if type(hists[0]) is r2m.Hist:
             if options.plotstyles_mpl[i] == "stack":
                 barstack.add(hist, log=options.logy,
                              edgecolor=options.linecolors[i],
@@ -541,35 +538,43 @@ def process_hist_mpl(path, name, counter, options):
                 histmax = max(histmax, barstack.max())
             else:
                 histmax = max(histmax, max(hist))
-        fullstack.add(hist)
-        hists.append(hist)
+            fullstack.add(hist)
     if allempty:
         plt.figtext(0.5, 0.5, "No Entries", ha='center', va='center')
-    elif dimension == 1:
-        for i, (file, hist) in enumerate(zip(files, hists)):
-            if options.plotstyles_mpl[i] == "errorbar":
-                if options.logy:
-                    # Logy would fail if hist all zeroes
-                    if np.nonzero(hist.y)[0].tolist():
-                        plt.yscale('log')
-                hist.errorbar(yerr=True, fmt=options.markers_mpl[i],
-                              color=options.fillcolors[i],
-                              ecolor=options.errorbarcolors_mpl[i])
-            if options.plotstyles_mpl[i] == "hist":
-                for j in range(hist.nbins):
-                    hist.y[j] = max(hist.y[j], 1e-10)
-                hist.hist(alpha=options.alphas_mpl[i], histtype='stepfilled',
-                          log=options.logy, edgecolor=options.linecolors[i],
-                          facecolor=options.fillcolors[i])
-            if options.plotstyles_mpl[i] == "bar":
-                hist.bar(alpha=options.alphas_mpl[i], log=options.logy,
-                         edgecolor=options.linecolors[i],
-                         facecolor=options.fillcolors[i])
-            if options.logx:
-                for ax in axes_list:
-                    ax.set_xscale('log')
+    elif type(hists[0]) is r2m.Hist:
+        for i, hist in enumerate(hists):
+            if hist:
+                if options.plotstyles_mpl[i] == "errorbar":
+                    if options.logy:
+                        # Logy would fail if hist all zeroes
+                        if np.nonzero(hist.y)[0].tolist():
+                            plt.yscale('log')
+                    hist.errorbar(yerr=True, fmt=options.markers_mpl[i],
+                                  color=options.fillcolors[i],
+                                  ecolor=options.errorbarcolors_mpl[i],
+                                  label_rotation=options.xlabel_rotation,
+                                  label_alignment=options.xlabel_alignment)
+                if options.plotstyles_mpl[i] == "hist":
+                    for j in range(hist.nbins):
+                        hist.y[j] = max(hist.y[j], 1e-10)
+                    hist.hist(alpha=options.alphas_mpl[i],
+                              histtype='stepfilled',
+                              log=options.logy, edgecolor=options.linecolors[i],
+                              facecolor=options.fillcolors[i],
+                              label_rotation=options.xlabel_rotation,
+                              label_alignment=options.xlabel_alignment)
+                if options.plotstyles_mpl[i] == "bar":
+                    hist.bar(alpha=options.alphas_mpl[i], log=options.logy,
+                             edgecolor=options.linecolors[i],
+                             facecolor=options.fillcolors[i],
+                             label_rotation=options.xlabel_rotation,
+                             label_alignment=options.xlabel_alignment)
+                if options.logx:
+                    for ax in axes_list:
+                        ax.set_xscale('log')
         if barstack.hists:
-            barstack.barstack()
+            barstack.barstack(label_rotation=options.xlabel_rotation,
+                              label_alignment=options.xlabel_alignment)
         plt.xlim(hist.xedges[0], hist.xedges[-1])
         if options.logy and 'hist' in options.plotstyles_mpl:
             my_min = fullstack.min(threshold=1.1e-10)
@@ -605,19 +610,23 @@ def process_hist_mpl(path, name, counter, options):
             except ValueError:
                 pass
             plt.legend(numpoints=1, loc=options.legend)
-    else:
+    else: # 2D hists
         drawfunc = getattr(hist, options.mpl2D)
         if 'col' in options.mpl2D:
             drawfunc()
         else:
             drawfunc(color=options.fillcolors[0])
-    plt.title(fullstack[-1].title)
-    plt.ylabel(fullstack[-1].ylabel)
+    plt.title(hists[0].title)
+    plt.ylabel(hists[0].ylabel)
     fig.sca(axes_list[1])
-    plt.xlabel(fullstack[-1].xlabel)
+    plt.xlabel(hists[0].xlabel)
     if options.ratio:
         fig.sca(axes_list[1])
         plot_ratio_mpl(hists, options)
+    if options.label_text:
+        plt.figtext(options.label_location[0], options.label_location[1],
+                    options.label_text, ha=options.label_align,
+                    fontsize=options.label_fontsize)
     if options.numbering:
         plt.figtext(options.numbering_x_mpl, options.numbering_y_mpl,
                     counter, size=options.numbering_size_mpl,
@@ -632,9 +641,10 @@ def process_hist_mpl(path, name, counter, options):
 def open_rootfiles(filenames):
     """Return a list of RootFile objects from a list of file names."""
     try:
-        return [RootFile(name) for name in filenames]
+        return [r2m.RootFile(name) for name in filenames]
     except TypeError:
-        return [RootFile(name, displayname) for name, displayname in filenames]
+        return [r2m.RootFile(name, displayname)
+                for name, displayname in filenames]
 
 def parse_size(size_option):
     """Return a width and height parsed from size_option."""
@@ -832,11 +842,13 @@ def plot_ratio_mpl(hists, options):
     for i in range(len(hists)):
         if i != ratio_index:
             ratio_hist = hists[i] / hists[ratio_index]
+            ratio_hist.y = [item or 0 for item in ratio_hist.y]
+            average = sum(ratio_hist.y) / ratio_hist.nbins
             for j in range(ratio_hist.nbins):
-                average = sum(ratio_hist.y) / ratio_hist.nbins
                 if ratio_hist.y[j] > options.ratio_cutoff * average:
-                    ratio_hist.y[j] = None
-                    ratio_hist.yerr[j] = None
+                    ratio_hist.y[j] = 0
+                    ratio_hist.yerr[0][j] = 0
+                    ratio_hist.yerr[1][j] = 0
             stack.add(ratio_hist, fmt=options.markers_mpl[i],
                   color=options.fillcolors[i],
                   ecolor=options.errorbarcolors_mpl[i])
@@ -860,6 +872,8 @@ def wilson(numerator, denominator):
 
 def make_html_index(base_dir, directory, filetype, stylesheet, ncolumns=2):
     fullpath = '%s%s' % (base_dir, directory)
+    if not os.path.exists(fullpath):
+        os.makedirs(fullpath)
     output = open('%s/index.html' % fullpath, 'w')
     contents = sorted(os.listdir(fullpath))
     subdirs = fullpath.split('/')
@@ -958,6 +972,7 @@ def add_options_from_configuration(options):
     options = append_to_options(default_rootplot_config, options)
     rc_path = os.path.expanduser('~/.rootplot')
     if os.path.exists(rc_path):
+        print "Using styles and options from ~/.rootplot"
         shutil.copy(rc_path, configdir + '/rootplotrc.py')
         options.config_files.insert(0, 'rootplotrc.py')
     for f in options.config_files:
@@ -990,7 +1005,7 @@ def process_options(options):
                   "which is required for these options")
             sys.exit(1)
     else:
-        global r2m, np, plt
+        global np, plt
         if options.ext == 'png':
             matplotlib.use('AGG')
         if options.ext == 'pdf':
@@ -1012,10 +1027,9 @@ def process_options(options):
                 sys.exit(1)
         if options.mpl:
             try:
-                import root2matplotlib as r2m
                 import numpy as np
             except ImportError:
-                print("Unable to load root2matplotlib, "
+                print("Unable to load numpy, "
                       "which is required for operation with matplotlib")
                 sys.exit(1)
     if options.normalize:
@@ -1080,21 +1094,42 @@ def process_options(options):
                                for x in colors])
     return options
 
+def get_plot_specific_options(histname, dict_options):
+    matching_options = []
+    for key in dict_options.keys():
+        for string in dict_options[key]:
+            if string and string in histname:
+                matching_options.append(key)
+                break
+    return matching_options
+
 def process_local_options(options, name):
     """Turn on sticky options for a particular plot."""
+    new_options = copy.copy(options)
     if options.sticky:
-        if 'Norm' in name: options.area_normalize = True
-        if 'Eff' in name: options.efficiency = True
-        if 'Logx' in name: options.logx = True
-        if 'Logy' in name: options.logy = True
-        if 'Zero' in name: options.zero = True
-	if 'Overflow' in name: options.overflow = True
-        if 'Underflow' in name: options.underflow = True
-        if 'Ratio' in name: options.ratio = True
-    if options.logy:
-        options.zero = False
-        options.top_padding_factor = options.top_padding_factor_log
-    return options
+        if 'Norm' in name: new_options.area_normalize = True
+        if 'Eff' in name: new_options.efficiency = True
+        if 'Logx' in name: new_options.logx = True
+        if 'Logy' in name: new_options.logy = True
+        if 'Zero' in name: new_options.zero = True
+	if 'Overflow' in name: new_options.overflow = True
+        if 'Underflow' in name: new_options.underflow = True
+        if 'Ratio' in name: new_options.ratio = True
+    specific_options = get_plot_specific_options(name,
+                                                 options.options_by_histname)
+    if specific_options:
+        if 'norm' in specific_options: new_options.area_normalize = True
+        if 'efficiency' in specific_options: new_options.efficiency = True
+        if 'logx' in specific_options: new_options.logx = True
+        if 'logy' in specific_options: new_options.logy = True
+        if 'zero' in specific_options: new_options.zero = True
+        if 'overflow' in specific_options: new_options.overflow = True
+        if 'underflow' in specific_options: new_options.underflow = True
+        if 'ratio' in specific_options: new_options.ratio = True
+    if new_options.logy:
+        new_options.zero = False
+        new_options.top_padding_factor = options.top_padding_factor_log
+    return new_options
 
 
 ######## Define the main program #############################################
@@ -1285,6 +1320,7 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     filenames_from_interface = [x for x in arguments if x[-5:] == ".root"]
     ## add style options and create the canvas
     options = add_options_from_configuration(options)
+    options.top_padding_factor_cfg = options.top_padding_factor # store cfg default
     if "filenames" in dir(options):
         if filenames_from_interface:
             print "Warning: filenames parameter defined in configuration;"
